//CNC router hardware revision 2.1
//
//Code to run on the command computer see:
//commands-CNC.c
//
//Source & info: www.HomoFaciens.de


//#include <avr/io.h>
//#include <util/delay.h>

//Change values below to adapt your motor

#define X_STEP       2       //Step input A4988
#define X_DIR        5       //Direction input A4988

#define Y_STEP       3       //Step input A4988
#define Y_DIR        6       //Direction input A4988

#define Z_STEP       4       //Step input A4988
#define Z_DIR        7       //Direction input A4988

#define DISABLE      8       //DISABLE A49881

#define RELAY       11       //Router relay

#define READ_LENGTH 10


int readySend;
byte readByte;

long XSetPoint;
long XIsPoint;

long YSetPoint;
long YIsPoint;

long ZSetPoint;
long ZIsPoint;

unsigned long stepTime;

unsigned long microsStart;
unsigned long microsTime;
unsigned long stepPause;


void establishContact() {
  while (Serial.available() <= 0) {
    Serial.print('X');   // send a capital X
    delay(300);
  }
}

void setup(){

  pinMode(DISABLE, OUTPUT);
  digitalWrite(DISABLE, 1);

  readByte = 0;
  readySend = 0;

  pinMode(RELAY, OUTPUT);
  digitalWrite(RELAY, 0);

  pinMode(X_STEP, OUTPUT);
  pinMode(X_DIR, OUTPUT);

  digitalWrite(X_STEP, 0);
  digitalWrite(X_DIR, 0);

  pinMode(Y_STEP, OUTPUT);
  pinMode(Y_DIR, OUTPUT);

  digitalWrite(Y_STEP, 0);
  digitalWrite(Y_DIR, 0);

  pinMode(Z_STEP, OUTPUT);
  pinMode(Z_DIR, OUTPUT);

  digitalWrite(Z_STEP, 0);
  digitalWrite(Z_DIR, 0);

  XSetPoint = 0L;
  XIsPoint  = 0L;

  YSetPoint = 0L;
  YIsPoint  = 0L;

  ZSetPoint = 0L;
  ZIsPoint  = 0L;

  stepPause = 1000;

  // start serial port at 115200 bps:
  Serial.begin(115200);
  establishContact();  // send a byte to establish contact until receiver responds

  digitalWrite(DISABLE, 0);


}


void loop(){

  if(XSetPoint > XIsPoint){
    digitalWrite(X_DIR, 0);
    digitalWrite(X_STEP, 1);
    XIsPoint++;
  }
  if(XSetPoint < XIsPoint){
    digitalWrite(X_DIR, 1);
    digitalWrite(X_STEP, 1);
    XIsPoint--;
  }

  if(YSetPoint > YIsPoint){
    digitalWrite(Y_DIR, 0);
    digitalWrite(Y_STEP, 1);
    YIsPoint++;
  }
  if(YSetPoint < YIsPoint){
    digitalWrite(Y_DIR, 1);
    digitalWrite(Y_STEP, 1);
    YIsPoint--;
  }

  if(ZSetPoint > ZIsPoint){
    digitalWrite(Z_DIR, 0);
    digitalWrite(Z_STEP, 1);
    ZIsPoint++;
  }
  if(ZSetPoint < ZIsPoint){
    digitalWrite(Z_DIR, 1);
    digitalWrite(Z_STEP, 1);
    ZIsPoint--;
  }


  delayMicroseconds(20);
  digitalWrite(X_STEP, 0);
  digitalWrite(Y_STEP, 0);
  digitalWrite(Z_STEP, 0);


  microsTime = microsStart;
  while(microsTime - microsStart < stepPause){
    microsTime = micros();
    if(microsTime < microsStart){
      microsStart = micros();
    }
  }
  microsStart = micros();


  readByte = 0;
  if (Serial.available() > 0){//if we get a valid byte, read from serial:
    //get incoming byte:
    readByte = Serial.read();
    readySend++;
    if(readySend == READ_LENGTH){
      Serial.print('r');   // send a 'r' to initiate next data from computer
      readySend = 0;
    }
  }//if (Serial.available() > 0){



  if(readByte > 0){
    if(readByte == 255){
      digitalWrite(RELAY, 1);
    }
    if(readByte == 254){
      digitalWrite(RELAY, 0);
    }

    if(readByte == 128){
      stepPause += 100;
    }

    if(readByte == 64){
      stepPause -= 100;
    }

    if(readByte == 32){
      ZSetPoint--;
    }
    if(readByte == 16){
      ZSetPoint++;
    }

    if(readByte == 8){
      YSetPoint--;
    }
    if(readByte == 4){
      YSetPoint++;
    }

    if(readByte == 2){
      XSetPoint--;
    }
    if(readByte == 1){
      XSetPoint++;
    }

  }//if(readByte > 0){


}


